# Copyright GlitterClient GmbH and GlitterClient contributors
# SPDX-License-Identifier: (Apache-2.0 AND CC-BY-4.0)
# Code is Apache-2.0 and docs are CC-BY-4.0

# from .driver import GlitterClient   # noqa
from .driver import GlitterClient   # noqa


__author__ = 'ted'
__email__ = 'ted@glitterprotocol.io'
__version__ = '0.1'
__release__ = '0.1.1'
